(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/platform-browser'), require('@angular/common')) :
    typeof define === 'function' && define.amd ? define('ngx-widget-grid', ['exports', '@angular/core', '@angular/platform-browser', '@angular/common'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global['ngx-widget-grid'] = {}, global.ng.core, global.ng.platformBrowser, global.ng.common));
}(this, (function (exports, core, platformBrowser, common) { 'use strict';

    var CellSize = /** @class */ (function () {
        function CellSize(rowCount, columnCount) {
            this._height = 0;
            this._width = 0;
            this._height = rowCount ? 100 / rowCount : 0;
            this._width = columnCount ? 100 / columnCount : 0;
        }
        ;
        Object.defineProperty(CellSize.prototype, "height", {
            get: function () {
                return this._height;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(CellSize.prototype, "width", {
            get: function () {
                return this._width;
            },
            enumerable: false,
            configurable: true
        });
        return CellSize;
    }());

    var Grid = /** @class */ (function () {
        function Grid(rows, columns) {
            this._widgets = [];
            this._rows = 3;
            this._columns = 3;
            if (+rows) {
                this._rows = +rows;
            }
            if (+columns) {
                this._columns = +columns;
            }
            this._cellSize = new CellSize(this._rows, this._columns);
        }
        Object.defineProperty(Grid.prototype, "widgets", {
            get: function () {
                return this._widgets;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(Grid.prototype, "rows", {
            get: function () {
                return this._rows;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(Grid.prototype, "columns", {
            get: function () {
                return this._columns;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(Grid.prototype, "cellSize", {
            get: function () {
                return this._cellSize;
            },
            enumerable: false,
            configurable: true
        });
        Grid.prototype.add = function (widget) {
            this._widgets.push(widget);
        };
        Grid.prototype.remove = function (widget) {
            var widgetIndex = this._widgets.indexOf(widget);
            if (widgetIndex > -1) {
                this._widgets.splice(widgetIndex, 1);
            }
        };
        Grid.prototype.removeAll = function () {
            this._widgets.splice(0);
        };
        Grid.prototype.hasWidget = function (widget) {
            var widgetIndex = this._widgets.indexOf(widget);
            return widgetIndex > -1;
        };
        Grid.prototype.resize = function (rows, columns) {
            columns = +columns || 0;
            rows = +rows || 0;
            if (columns > 0 && rows > 0 && columns !== this._columns || rows !== this._rows) {
                this._columns = columns;
                this._rows = rows;
                this._cellSize = new CellSize(this._rows, this._columns);
            }
        };
        return Grid;
    }());

    var Rectangle = /** @class */ (function () {
        function Rectangle(obj) {
            this.top = 0;
            this.left = 0;
            this.width = 0;
            this.height = 0;
            if (obj) {
                this.top = +obj.top || 0;
                this.left = +obj.left || 0;
                this.width = +obj.width || 0;
                this.height = +obj.height || 0;
            }
        }
        Object.defineProperty(Rectangle.prototype, "bottom", {
            get: function () {
                return this.top + this.height - 1;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(Rectangle.prototype, "right", {
            get: function () {
                return this.left + this.width - 1;
            },
            enumerable: false,
            configurable: true
        });
        Rectangle.prototype.getSurfaceArea = function () {
            return this.height * this.width;
        };
        return Rectangle;
    }());

    var WidgetConfig = /** @class */ (function () {
        function WidgetConfig(rect) {
            this._position = new Rectangle();
            this.id = this.generateUID();
            if (rect) {
                this.position = rect;
            }
        }
        Object.defineProperty(WidgetConfig.prototype, "position", {
            get: function () {
                return this._position;
            },
            set: function (gridArea) {
                this._position.top = +gridArea.top ? +gridArea.top : 0;
                this._position.left = +gridArea.left ? +gridArea.left : 0;
                this._position.width = +gridArea.width ? +gridArea.width : 0;
                this._position.height = +gridArea.height ? +gridArea.height : 0;
            },
            enumerable: false,
            configurable: true
        });
        WidgetConfig.prototype.generateUID = function () {
            return 'ngxDashboardWidget-' + ++WidgetConfig.widgetCount;
        };
        return WidgetConfig;
    }());
    WidgetConfig.widgetCount = 0;

    var Utils = /** @class */ (function () {
        function Utils() {
        }
        Utils.isNumber = function (val) {
            return typeof val === 'number';
        };
        Utils.isDefined = function (val) {
            return val !== undefined;
        };
        Utils.isObject = function (val) {
            return typeof val === 'object';
        };
        return Utils;
    }());
    var RESIZE_DIRECTIONS;
    (function (RESIZE_DIRECTIONS) {
        RESIZE_DIRECTIONS["topLeft"] = "NW";
        RESIZE_DIRECTIONS["top"] = "N";
        RESIZE_DIRECTIONS["topRight"] = "NE";
        RESIZE_DIRECTIONS["right"] = "E";
        RESIZE_DIRECTIONS["bottomRight"] = "SE";
        RESIZE_DIRECTIONS["bottom"] = "S";
        RESIZE_DIRECTIONS["bottomLeft"] = "SW";
        RESIZE_DIRECTIONS["left"] = "W";
    })(RESIZE_DIRECTIONS || (RESIZE_DIRECTIONS = {}));
    var ALL_DIRECTIONS = [
        RESIZE_DIRECTIONS.bottom,
        RESIZE_DIRECTIONS.left,
        RESIZE_DIRECTIONS.right,
        RESIZE_DIRECTIONS.top,
        RESIZE_DIRECTIONS.bottomLeft,
        RESIZE_DIRECTIONS.bottomRight,
        RESIZE_DIRECTIONS.topLeft,
        RESIZE_DIRECTIONS.topRight
    ];

    var NgxWidgetComponent = /** @class */ (function () {
        function NgxWidgetComponent(elRef) {
            this.elRef = elRef;
            this.swapOnMove = false;
            this.positionChange = new core.EventEmitter();
            this.movable = false;
            this.allDirections = RESIZE_DIRECTIONS;
            this.isTopResizable = false;
            this.isRightResizable = false;
            this.isBottomResizable = false;
            this.isLeftResizable = false;
            this.isTopRightResizable = false;
            this.isTopLeftResizable = false;
            this.isBottomRightResizable = false;
            this.isBottomLeftResizable = false;
            this._resizable = false;
            this._resizeDirections = ALL_DIRECTIONS;
            this.widgetConfig = new WidgetConfig(this.position);
        }
        Object.defineProperty(NgxWidgetComponent.prototype, "position", {
            get: function () {
                return this._position;
            },
            set: function (position) {
                this._position = position;
                this.widgetConfig.position = position;
                this.positionChange.emit(position);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgxWidgetComponent.prototype, "resizable", {
            get: function () {
                return this._resizable;
            },
            set: function (resizable) {
                this._resizable = resizable;
                if (resizable) {
                    this.setResizeDirections();
                }
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgxWidgetComponent.prototype, "resizeDirections", {
            get: function () {
                return this._resizeDirections;
            },
            set: function (dirs) {
                this._resizeDirections = dirs.filter(function (dir) {
                    return ALL_DIRECTIONS.indexOf(dir.toUpperCase()) !== -1;
                });
                this.setResizeDirections();
            },
            enumerable: false,
            configurable: true
        });
        NgxWidgetComponent.prototype.setResizeDirections = function () {
            var _this = this;
            this.isTopResizable = false;
            this.isRightResizable = false;
            this.isBottomResizable = false;
            this.isLeftResizable = false;
            this.isTopRightResizable = false;
            this.isTopLeftResizable = false;
            this.isBottomRightResizable = false;
            this.isBottomLeftResizable = false;
            this._resizeDirections.forEach(function (dir) {
                switch (dir) {
                    case RESIZE_DIRECTIONS.top:
                        _this.isTopResizable = true;
                        break;
                    case RESIZE_DIRECTIONS.left:
                        _this.isLeftResizable = true;
                        break;
                    case RESIZE_DIRECTIONS.bottom:
                        _this.isBottomResizable = true;
                        break;
                    case RESIZE_DIRECTIONS.right:
                        _this.isRightResizable = true;
                        break;
                    case RESIZE_DIRECTIONS.topLeft:
                        _this.isTopLeftResizable = true;
                        break;
                    case RESIZE_DIRECTIONS.topRight:
                        _this.isTopRightResizable = true;
                        break;
                    case RESIZE_DIRECTIONS.bottomLeft:
                        _this.isBottomLeftResizable = true;
                        break;
                    case RESIZE_DIRECTIONS.bottomRight:
                        _this.isBottomRightResizable = true;
                        break;
                    default:
                }
            });
        };
        NgxWidgetComponent.prototype.getConfig = function () {
            return this.widgetConfig;
        };
        NgxWidgetComponent.prototype.getEl = function () {
            return this.elRef;
        };
        return NgxWidgetComponent;
    }());
    NgxWidgetComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'ngx-widget',
                    template: "<div class=\"wg-widget-content\">\n  <ng-content></ng-content>\n</div>\n<div [ngxWidgetMover]=\"swapOnMove\"\n     *ngIf=\"movable\"\n     class=\"wg-widget-edit-move\">\n</div>\n<div *ngIf=\"resizable\"\n     class=\"wg-widget-edit-resize\">\n  <div class=\"wg-resize wg-resize-axis wg-resize-n\"\n       [ngxWidgetResizer]=\"allDirections.top\"\n       [hidden]=\"!isTopResizable\"></div>\n  <div class=\"wg-resize wg-resize-axis wg-resize-e\"\n       [ngxWidgetResizer]=\"allDirections.right\"\n       [hidden]=\"!isRightResizable\"></div>\n  <div class=\"wg-resize wg-resize-axis wg-resize-s\"\n       [ngxWidgetResizer]=\"allDirections.bottom\"\n       [hidden]=\"!isBottomResizable\"></div>\n  <div class=\"wg-resize wg-resize-axis wg-resize-w\"\n       [ngxWidgetResizer]=\"allDirections.left\"\n       [hidden]=\"!isLeftResizable\"></div>\n  <div class=\"wg-resize wg-resize-diag wg-resize-nw\"\n       [ngxWidgetResizer]=\"allDirections.topLeft\"\n       [hidden]=\"!isTopLeftResizable\"></div>\n  <div class=\"wg-resize wg-resize-diag wg-resize-ne\"\n       [ngxWidgetResizer]=\"allDirections.topRight\"\n       [hidden]=\"!isTopRightResizable\"></div>\n  <div class=\"wg-resize wg-resize-diag wg-resize-se\"\n       [ngxWidgetResizer]=\"allDirections.bottomRight\"\n       [hidden]=\"!isBottomRightResizable\"></div>\n  <div class=\"wg-resize wg-resize-diag wg-resize-sw\"\n       [ngxWidgetResizer]=\"allDirections.bottomLeft\"\n       [hidden]=\"!isBottomLeftResizable\"></div>\n</div>\n\n",
                    changeDetection: core.ChangeDetectionStrategy.OnPush,
                    styles: [":host{position:absolute}:host .wg-widget-content{position:absolute;overflow:hidden;height:100%;width:100%}:host.wg-moving .wg-widget-content,:host.wg-resizing .wg-widget-content{opacity:.2;z-index:18}:host.wg-moving .wg-widget-edit-move{cursor:move;cursor:-webkit-grabbing}.wg-widget-edit-move{position:absolute;top:6px;right:6px;bottom:6px;left:6px;z-index:20;touch-action:none;cursor:move;cursor:-webkit-grab}.wg-widget-edit-resize{position:absolute;touch-action:none;top:0;right:0;bottom:0;left:0;z-index:15}.wg-resize{position:absolute}.wg-resize-diag{border-color:rgba(0,113,188,.75);border-style:solid;width:0;height:0;z-index:1}.wg-resize-diag:hover{border-color:rgba(0,138,229,.75)}.wg-resize-diag.dragging{border-color:rgba(0,174,239,.75)}.wg-resize-axis{background-color:rgba(0,113,188,.12)}.wg-resize-axis.dragging{background-color:rgba(0,174,239,.24)}.wg-resize-nw{top:0;left:0;cursor:nw-resize;border-width:.5rem .5rem 0 0;border-right-color:transparent!important}.wg-resize-ne{top:0;right:0;cursor:ne-resize;border-width:.5rem 0 0 .5rem;border-left-color:transparent!important}.wg-resize-se{bottom:0;right:0;cursor:se-resize;border-width:0 0 .5rem .5rem;border-left-color:transparent!important}.wg-resize-sw{bottom:0;left:0;cursor:sw-resize;border-width:0 .5rem .5rem 0;border-right-color:transparent!important}.wg-resize-n{top:0;right:0;left:0;height:.25rem;cursor:n-resize}.wg-resize-e{top:0;right:0;bottom:0;width:.25rem;cursor:e-resize}.wg-resize-s{right:0;bottom:0;left:0;height:.25rem;cursor:s-resize}.wg-resize-w{top:0;bottom:0;left:0;width:.25rem;cursor:w-resize}[hidden]{display:none}"]
                },] }
    ];
    NgxWidgetComponent.ctorParameters = function () { return [
        { type: core.ElementRef }
    ]; };
    NgxWidgetComponent.propDecorators = {
        swapOnMove: [{ type: core.Input }],
        positionChange: [{ type: core.Output }],
        movable: [{ type: core.Input }],
        position: [{ type: core.Input }],
        resizable: [{ type: core.Input }],
        resizeDirections: [{ type: core.Input }]
    };

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */
    /* global Reflect, Promise */
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b)
                if (Object.prototype.hasOwnProperty.call(b, p))
                    d[p] = b[p]; };
        return extendStatics(d, b);
    };
    function __extends(d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    }
    var __assign = function () {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s)
                    if (Object.prototype.hasOwnProperty.call(s, p))
                        t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };
    function __rest(s, e) {
        var t = {};
        for (var p in s)
            if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
                t[p] = s[p];
        if (s != null && typeof Object.getOwnPropertySymbols === "function")
            for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
                if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                    t[p[i]] = s[p[i]];
            }
        return t;
    }
    function __decorate(decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
            r = Reflect.decorate(decorators, target, key, desc);
        else
            for (var i = decorators.length - 1; i >= 0; i--)
                if (d = decorators[i])
                    r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    }
    function __param(paramIndex, decorator) {
        return function (target, key) { decorator(target, key, paramIndex); };
    }
    function __metadata(metadataKey, metadataValue) {
        if (typeof Reflect === "object" && typeof Reflect.metadata === "function")
            return Reflect.metadata(metadataKey, metadataValue);
    }
    function __awaiter(thisArg, _arguments, P, generator) {
        function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
        return new (P || (P = Promise))(function (resolve, reject) {
            function fulfilled(value) { try {
                step(generator.next(value));
            }
            catch (e) {
                reject(e);
            } }
            function rejected(value) { try {
                step(generator["throw"](value));
            }
            catch (e) {
                reject(e);
            } }
            function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
            step((generator = generator.apply(thisArg, _arguments || [])).next());
        });
    }
    function __generator(thisArg, body) {
        var _ = { label: 0, sent: function () { if (t[0] & 1)
                throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
        return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function () { return this; }), g;
        function verb(n) { return function (v) { return step([n, v]); }; }
        function step(op) {
            if (f)
                throw new TypeError("Generator is already executing.");
            while (_)
                try {
                    if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done)
                        return t;
                    if (y = 0, t)
                        op = [op[0] & 2, t.value];
                    switch (op[0]) {
                        case 0:
                        case 1:
                            t = op;
                            break;
                        case 4:
                            _.label++;
                            return { value: op[1], done: false };
                        case 5:
                            _.label++;
                            y = op[1];
                            op = [0];
                            continue;
                        case 7:
                            op = _.ops.pop();
                            _.trys.pop();
                            continue;
                        default:
                            if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) {
                                _ = 0;
                                continue;
                            }
                            if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) {
                                _.label = op[1];
                                break;
                            }
                            if (op[0] === 6 && _.label < t[1]) {
                                _.label = t[1];
                                t = op;
                                break;
                            }
                            if (t && _.label < t[2]) {
                                _.label = t[2];
                                _.ops.push(op);
                                break;
                            }
                            if (t[2])
                                _.ops.pop();
                            _.trys.pop();
                            continue;
                    }
                    op = body.call(thisArg, _);
                }
                catch (e) {
                    op = [6, e];
                    y = 0;
                }
                finally {
                    f = t = 0;
                }
            if (op[0] & 5)
                throw op[1];
            return { value: op[0] ? op[1] : void 0, done: true };
        }
    }
    var __createBinding = Object.create ? (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        Object.defineProperty(o, k2, { enumerable: true, get: function () { return m[k]; } });
    }) : (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        o[k2] = m[k];
    });
    function __exportStar(m, o) {
        for (var p in m)
            if (p !== "default" && !Object.prototype.hasOwnProperty.call(o, p))
                __createBinding(o, m, p);
    }
    function __values(o) {
        var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
        if (m)
            return m.call(o);
        if (o && typeof o.length === "number")
            return {
                next: function () {
                    if (o && i >= o.length)
                        o = void 0;
                    return { value: o && o[i++], done: !o };
                }
            };
        throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }
    function __read(o, n) {
        var m = typeof Symbol === "function" && o[Symbol.iterator];
        if (!m)
            return o;
        var i = m.call(o), r, ar = [], e;
        try {
            while ((n === void 0 || n-- > 0) && !(r = i.next()).done)
                ar.push(r.value);
        }
        catch (error) {
            e = { error: error };
        }
        finally {
            try {
                if (r && !r.done && (m = i["return"]))
                    m.call(i);
            }
            finally {
                if (e)
                    throw e.error;
            }
        }
        return ar;
    }
    function __spread() {
        for (var ar = [], i = 0; i < arguments.length; i++)
            ar = ar.concat(__read(arguments[i]));
        return ar;
    }
    function __spreadArrays() {
        for (var s = 0, i = 0, il = arguments.length; i < il; i++)
            s += arguments[i].length;
        for (var r = Array(s), k = 0, i = 0; i < il; i++)
            for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
                r[k] = a[j];
        return r;
    }
    ;
    function __await(v) {
        return this instanceof __await ? (this.v = v, this) : new __await(v);
    }
    function __asyncGenerator(thisArg, _arguments, generator) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var g = generator.apply(thisArg, _arguments || []), i, q = [];
        return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
        function verb(n) { if (g[n])
            i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
        function resume(n, v) { try {
            step(g[n](v));
        }
        catch (e) {
            settle(q[0][3], e);
        } }
        function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
        function fulfill(value) { resume("next", value); }
        function reject(value) { resume("throw", value); }
        function settle(f, v) { if (f(v), q.shift(), q.length)
            resume(q[0][0], q[0][1]); }
    }
    function __asyncDelegator(o) {
        var i, p;
        return i = {}, verb("next"), verb("throw", function (e) { throw e; }), verb("return"), i[Symbol.iterator] = function () { return this; }, i;
        function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: n === "return" } : f ? f(v) : v; } : f; }
    }
    function __asyncValues(o) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var m = o[Symbol.asyncIterator], i;
        return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
        function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
        function settle(resolve, reject, d, v) { Promise.resolve(v).then(function (v) { resolve({ value: v, done: d }); }, reject); }
    }
    function __makeTemplateObject(cooked, raw) {
        if (Object.defineProperty) {
            Object.defineProperty(cooked, "raw", { value: raw });
        }
        else {
            cooked.raw = raw;
        }
        return cooked;
    }
    ;
    var __setModuleDefault = Object.create ? (function (o, v) {
        Object.defineProperty(o, "default", { enumerable: true, value: v });
    }) : function (o, v) {
        o["default"] = v;
    };
    function __importStar(mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k))
                    __createBinding(result, mod, k);
        __setModuleDefault(result, mod);
        return result;
    }
    function __importDefault(mod) {
        return (mod && mod.__esModule) ? mod : { default: mod };
    }
    function __classPrivateFieldGet(receiver, privateMap) {
        if (!privateMap.has(receiver)) {
            throw new TypeError("attempted to get private field on non-instance");
        }
        return privateMap.get(receiver);
    }
    function __classPrivateFieldSet(receiver, privateMap, value) {
        if (!privateMap.has(receiver)) {
            throw new TypeError("attempted to set private field on non-instance");
        }
        privateMap.set(receiver, value);
        return value;
    }

    var Cell = /** @class */ (function () {
        function Cell(top, left) {
            this._top = top;
            this._left = left;
        }
        Object.defineProperty(Cell.prototype, "top", {
            get: function () {
                return this._top;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(Cell.prototype, "left", {
            get: function () {
                return this._left;
            },
            enumerable: false,
            configurable: true
        });
        return Cell;
    }());

    var GridRenderer = /** @class */ (function () {
        function GridRenderer(grid) {
            this.positions = {};
            this.obstructions = [];
            this.grid = grid || new Grid();
        }
        Object.defineProperty(GridRenderer.prototype, "grid", {
            get: function () {
                return this._grid;
            },
            set: function (grid) {
                this._grid = grid;
                this.positions = {};
                this.cachedNextPosition = undefined;
                this.obstructions = [];
                for (var i = 0; i < grid.rows * grid.columns; i++) {
                    this.obstructions[i] = null;
                }
            },
            enumerable: false,
            configurable: true
        });
        GridRenderer.prototype.rasterizeCoords = function (left, top, gridWidth, gridHeight) {
            left = Math.min(Math.max(left, 0), gridWidth - 1);
            top = Math.min(Math.max(top, 0), gridHeight - 1);
            var x = Math.floor(top / gridHeight * this.grid.rows) + 1;
            var y = Math.floor(left / gridWidth * this.grid.columns) + 1;
            return new Cell(x, y);
        };
        GridRenderer.prototype.getWidgetIdAt = function (i, j) {
            for (var widgetId in this.positions) {
                if (this.positions.hasOwnProperty(widgetId)) {
                    var position = this.positions[widgetId];
                    if (position.top <= i && i <= (position.top + position.height - 1) &&
                        position.left <= j && j <= (position.left + position.width - 1)) {
                        return widgetId;
                    }
                }
            }
            return null;
        };
        GridRenderer.prototype.getWidgetPosition = function (widgetId) {
            return this.positions[widgetId];
        };
        GridRenderer.prototype.setWidgetPosition = function (widgetId, newPosition, swappingPositions) {
            var currPosition = this.positions[widgetId];
            if (currPosition && !swappingPositions) {
                this.setObstructionValues(currPosition, null);
            }
            newPosition = new Rectangle({
                top: Utils.isNumber(newPosition.top) ? newPosition.top : currPosition.top,
                left: Utils.isNumber(newPosition.left) ? newPosition.left : currPosition.left,
                width: Utils.isNumber(newPosition.width) ? newPosition.width : currPosition.width,
                height: Utils.isNumber(newPosition.height) ? newPosition.height : currPosition.height
            });
            this.positions[widgetId] = newPosition;
            this.setObstructionValues(this.positions[widgetId], widgetId);
            this.cachedNextPosition = undefined;
        };
        GridRenderer.prototype.hasSpaceLeft = function () {
            var e_1, _a;
            try {
                for (var _b = __values(this.obstructions), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var obstruction = _c.value;
                    if (obstruction === null) {
                        return true;
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
            return false;
        };
        GridRenderer.prototype.getNextPosition = function () {
            if (this.cachedNextPosition !== undefined) {
                return this.cachedNextPosition;
            }
            if (!this.hasSpaceLeft()) {
                return null;
            }
            var maxPosition = this.findLargestEmptyArea();
            this.cachedNextPosition = maxPosition;
            return maxPosition;
        };
        GridRenderer.prototype.isObstructed = function (i, j, excludedArea) {
            // obstructed if (i, j) exceeds the grid's regular non-expanding boundaries
            if (i < 1 || j < 1 || j > this.grid.columns || i > this.grid.rows) {
                return true;
            }
            var bottom = excludedArea && excludedArea.top + excludedArea.height - 1;
            var right = excludedArea && excludedArea.left + excludedArea.width - 1;
            // pass if (i, j) is within the excluded area, if any
            if (excludedArea &&
                excludedArea.top <= i &&
                i <= bottom &&
                excludedArea.left <= j &&
                j <= right) {
                return false;
            }
            return this._isObstructed(i, j);
        };
        GridRenderer.prototype._isObstructed = function (i, j) {
            return this.obstructions[(i - 1) * this.grid.columns + (j - 1)] !== null;
        };
        GridRenderer.prototype.isAreaObstructed = function (area, options) {
            if (!area) {
                return false;
            }
            options = Utils.isObject(options) ? options : {};
            var top = area.top;
            var left = area.left;
            var bottom = area.bottom || area.top + area.height - 1;
            var right = area.right || area.left + area.width - 1;
            if (!Utils.isNumber(top) || !Utils.isNumber(left) || !Utils.isNumber(bottom) || !Utils.isNumber(right)) {
                return false;
            }
            var verticalStart = options.fromBottom ? bottom : top;
            var verticalStep = options.fromBottom ? -1 : 1;
            var verticalEnd = (options.fromBottom ? top : bottom) + verticalStep;
            var horizontalStart = options.fromRight ? right : left;
            var horizontalStep = options.fromRight ? -1 : 1;
            var horizontalEnd = (options.fromRight ? left : right) + horizontalStep;
            for (var i = verticalStart; i !== verticalEnd; i += verticalStep) {
                for (var j = horizontalStart; j !== horizontalEnd; j += horizontalStep) {
                    if (this.isObstructed(i, j, options.excludedArea)) {
                        return true;
                    }
                }
            }
            return false;
        };
        GridRenderer.prototype.getStyle = function (widgetId) {
            var render = this.positions[widgetId];
            if (!render) {
                return { display: 'none' };
            }
            return {
                top: ((render.top - 1) * this.grid.cellSize.height).toString() + '%',
                height: (render.height * this.grid.cellSize.height).toString() + '%',
                left: ((render.left - 1) * this.grid.cellSize.width).toString() + '%',
                width: (render.width * this.grid.cellSize.width).toString() + '%'
            };
        };
        GridRenderer.prototype.setObstructionValues = function (area, value) {
            for (var i = area.top - 1; i < area.top + area.height - 1; i++) {
                for (var j = area.left - 1; j < area.left + area.width - 1; j++) {
                    this.obstructions[i * this.grid.columns + j] = value;
                }
            }
        };
        GridRenderer.prototype.findLargestEmptyArea = function () {
            var maxArea = null;
            var currMaxArea = null;
            var maxSurfaceArea = 0;
            var currMaxSurfaceArea = 0;
            for (var i = 1; i <= this.grid.rows; i++) {
                for (var j = 1; j <= this.grid.columns; j++) {
                    if (this._isObstructed(i, j)) {
                        continue;
                    }
                    var currAreaLimit = (this.grid.rows - i + 1) * (this.grid.columns - j + 1);
                    if (currAreaLimit < maxSurfaceArea) {
                        break;
                    }
                    currMaxArea = this._findLargestEmptyAreaFrom(new Cell(i, j));
                    currMaxSurfaceArea = currMaxArea.getSurfaceArea();
                    if (currMaxSurfaceArea > maxSurfaceArea) {
                        maxSurfaceArea = currMaxSurfaceArea;
                        maxArea = currMaxArea;
                    }
                }
            }
            return maxArea;
        };
        GridRenderer.prototype._findLargestEmptyAreaFrom = function (start) {
            if (!Utils.isDefined(this.grid) || !Utils.isNumber(this.grid.columns) || !Utils.isNumber(this.grid.rows)) {
                return null;
            }
            var maxArea = null, maxSurfaceArea = 0, endColumn = this.grid.columns;
            for (var i = start.top; i <= this.grid.rows; i++) {
                for (var j = start.left; j <= endColumn; j++) {
                    if (this._isObstructed(i, j)) {
                        endColumn = j - 1;
                        continue;
                    }
                    var currHeight = (i - start.top + 1);
                    var currWidth = (j - start.left + 1);
                    var currSurfaceArea = currHeight * currWidth;
                    if (currSurfaceArea > maxSurfaceArea) {
                        maxSurfaceArea = currSurfaceArea;
                        maxArea = new Rectangle({
                            top: start.top,
                            left: start.left,
                            width: currWidth,
                            height: currHeight
                        });
                    }
                }
            }
            return maxArea;
        };
        GridRenderer.prototype.render = function (grid, emitWidgetPositionUpdated) {
            var _this = this;
            this.grid = grid;
            var widgets = this.grid && this.grid.widgets ? this.grid.widgets : [];
            var unpositionedWidgets = [];
            widgets.forEach(function (widget) {
                var position = widget.position;
                if (position.width * position.height === 0 ||
                    _this.isAreaObstructed(position)) {
                    unpositionedWidgets.push(widget);
                }
                else {
                    _this.setWidgetPosition(widget.id, position, false);
                }
            });
            unpositionedWidgets.forEach(function (widget) {
                var nextPosition = _this.getNextPosition();
                if (nextPosition !== null) {
                    widget.position = nextPosition;
                    _this.setWidgetPosition(widget.id, nextPosition, false);
                }
                else {
                    widget.position = new Rectangle();
                    _this.setWidgetPosition(widget.id, new Rectangle(), false);
                }
                if (emitWidgetPositionUpdated) {
                    emitWidgetPositionUpdated(widget);
                }
            });
        };
        return GridRenderer;
    }());

    var NgxWidgetGridComponent = /** @class */ (function () {
        function NgxWidgetGridComponent(el, _changeDetector, _renderer) {
            this.el = el;
            this._changeDetector = _changeDetector;
            this._renderer = _renderer;
            this.showGrid = false;
            this.widgetPositionChange = new core.EventEmitter();
            this.gridFull = new core.EventEmitter();
            this.gridAlreadyFull = false;
            this._highlightNextPosition = false;
            this.grid = new Grid(this.rows, this.columns);
            this.gridRenderer = new GridRenderer(this.grid);
        }
        Object.defineProperty(NgxWidgetGridComponent.prototype, "rows", {
            get: function () {
                return this._rows;
            },
            set: function (rows) {
                this._rows = rows;
                this.updateGridSize();
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgxWidgetGridComponent.prototype, "columns", {
            get: function () {
                return this._columns;
            },
            set: function (cols) {
                this._columns = cols;
                this.updateGridSize();
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgxWidgetGridComponent.prototype, "highlightNextPosition", {
            get: function () {
                return this._highlightNextPosition;
            },
            set: function (highlightNext) {
                this._highlightNextPosition = highlightNext;
                if (highlightNext) {
                    this.updateNextPositionHighlight();
                }
                else {
                    this.resetHighlights();
                }
            },
            enumerable: false,
            configurable: true
        });
        NgxWidgetGridComponent.prototype.ngAfterViewInit = function () {
            var _this = this;
            this.refreshWidgets();
            this.widgetComponents.changes.subscribe(function () {
                _this.clearGrid();
                _this.refreshWidgets();
            });
        };
        NgxWidgetGridComponent.prototype.refreshWidgets = function () {
            var _this = this;
            this.widgetComponents.forEach(function (widget) {
                if (!_this.hasWidget(widget)) {
                    _this.addWidget(widget, true);
                }
                else {
                }
            });
            this.updateRendering();
        };
        NgxWidgetGridComponent.prototype.hasWidget = function (widget) {
            return this.grid.hasWidget(widget.getConfig());
        };
        NgxWidgetGridComponent.prototype.addWidget = function (widget, deferredRender) {
            this.grid.add(widget.getConfig());
            if (!deferredRender) {
                this.updateRendering();
            }
        };
        NgxWidgetGridComponent.prototype.clearGrid = function () {
            this.grid.removeAll();
        };
        NgxWidgetGridComponent.prototype.updateGridSize = function () {
            var columns = this.columns;
            var rows = this.rows;
            if (this.grid.columns !== columns || this.grid.rows !== rows) {
                this.grid.resize(rows, columns);
                this.updateRendering();
            }
        };
        NgxWidgetGridComponent.prototype.updateRendering = function () {
            var _this = this;
            this.gridRenderer.render(this.grid, this.emitUpdatePosition.bind(this));
            this.updateNextPositionHighlight();
            // TODO: retrieve all widgets and call their updateRendering
            if (this.widgetComponents) {
                this.widgetComponents.forEach(function (widget) {
                    _this.updateWidget(widget, false);
                });
            }
        };
        NgxWidgetGridComponent.prototype.getGridRectangle = function () {
            var gridContainer = this.el.nativeElement;
            // c.f. jQuery#offset: https://github.com/jquery/jquery/blob/2d715940b9b6fdeed005cd006c8bf63951cf7fb2/src/offset.js#L93-105
            var rect = gridContainer.getBoundingClientRect();
            if (rect.width || rect.height || gridContainer.getClientRects().length) {
                var doc = gridContainer.ownerDocument;
                var docElem = doc.documentElement;
                return new Rectangle({
                    top: rect.top + window.pageYOffset - docElem.clientTop,
                    left: rect.left + window.pageXOffset - docElem.clientLeft,
                    height: rect.height,
                    width: rect.width
                });
            }
            return new Rectangle({ top: 0, left: 0, height: 0, width: 0 });
        };
        NgxWidgetGridComponent.prototype.rasterizeCoords = function (x, y) {
            return this.gridRenderer.rasterizeCoords(x, y, this.el.nativeElement.clientWidth, this.el.nativeElement.clientHeight);
        };
        NgxWidgetGridComponent.prototype.updateWidget = function (widget, swappingWidgets) {
            var config = widget.getConfig();
            var newPosition = config.position;
            var el = widget.getEl();
            this.gridRenderer.setWidgetPosition(config.id, newPosition, swappingWidgets);
            var widgetStyles = this.getWidgetStyle(widget);
            for (var style in widgetStyles) {
                if (widgetStyles.hasOwnProperty(style)) {
                    this._renderer.setStyle(el.nativeElement, style, widgetStyles[style]);
                }
            }
            this.emitUpdatePosition(config);
            this.assessAvailableGridSpace();
        };
        NgxWidgetGridComponent.prototype.getWidgetPositionByWidgetId = function (widgetId) {
            return this.gridRenderer.getWidgetPosition(widgetId);
        };
        NgxWidgetGridComponent.prototype.getWidgetPosition = function (widget) {
            return this.getWidgetPositionByWidgetId(widget.getConfig().id);
        };
        NgxWidgetGridComponent.prototype.getWidgetStyle = function (widget) {
            return this.gridRenderer.getStyle(widget.getConfig().id);
        };
        NgxWidgetGridComponent.prototype.isPointObstructed = function (i, j) {
            return this.gridRenderer ? this.gridRenderer.isObstructed(i, j) : true;
        };
        NgxWidgetGridComponent.prototype.isAreaObstructed = function (area, options) {
            return this.gridRenderer ? this.gridRenderer.isAreaObstructed(area, options) : true;
        };
        NgxWidgetGridComponent.prototype.areaObstructor = function (area) {
            return this.gridRenderer.obstructions[(area.top - 1) * this.grid.columns + (area.left - 1)];
        };
        NgxWidgetGridComponent.prototype.highlightArea = function (area) {
            var _this = this;
            if (area.top && area.left && area.height && area.width) {
                setTimeout(function () {
                    _this.highlightedArea = area;
                    _this._changeDetector.markForCheck();
                });
            }
        };
        NgxWidgetGridComponent.prototype.updateNextPositionHighlight = function () {
            if (this.highlightNextPosition) {
                this.highlightedArea = this.gridRenderer.getNextPosition();
            }
        };
        NgxWidgetGridComponent.prototype.getNextPosition = function () {
            return this.gridRenderer.getNextPosition();
        };
        NgxWidgetGridComponent.prototype.getPositions = function () {
            return this.grid.widgets;
        };
        NgxWidgetGridComponent.prototype.resetHighlights = function () {
            var _this = this;
            setTimeout(function () {
                _this.highlightedArea = null;
                _this._changeDetector.markForCheck();
            });
        };
        NgxWidgetGridComponent.prototype.emitUpdatePosition = function (widget) {
            this.widgetPositionChange.emit({
                index: this.getWidgetIndex(widget),
                newPosition: widget.position
            });
        };
        NgxWidgetGridComponent.prototype.getWidgetIndex = function (widgetConfig) {
            for (var i = this.grid.widgets.length - 1; i >= 0; i--) {
                if (this.grid.widgets[i].id === widgetConfig.id) {
                    return i;
                }
            }
            return -1;
        };
        NgxWidgetGridComponent.prototype.getWidgetById = function (widgetId) {
            return this.widgetComponents.find(function (widgetCmp) {
                return widgetCmp.getConfig().id === widgetId;
            }) || null;
        };
        NgxWidgetGridComponent.prototype.assessAvailableGridSpace = function () {
            var gridHasSpaceLeft = this.gridRenderer.hasSpaceLeft();
            if (this.gridAlreadyFull) {
                if (gridHasSpaceLeft) {
                    this.gridFull.emit(false);
                    this.gridAlreadyFull = false;
                }
                else {
                    /*No change to grid status. was and still is full*/
                }
            }
            else {
                if (!gridHasSpaceLeft) {
                    this.gridFull.emit(true);
                    this.gridAlreadyFull = true;
                }
                else {
                    /*No change to grid status. had and still has available space*/
                }
            }
        };
        return NgxWidgetGridComponent;
    }());
    NgxWidgetGridComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'ngx-widget-grid',
                    template: "<ngx-grid-overlay [showGrid]=\"showGrid\"\n                  [renderer]=\"gridRenderer\"\n                  [highlight]=\"highlightedArea\"\n                  [cols]=\"columns\"\n                  [rows]=\"rows\">\n</ngx-grid-overlay>\n<div class=\"wg-grid-widgets\">\n  <ng-content></ng-content>\n</div>\n",
                    changeDetection: core.ChangeDetectionStrategy.OnPush,
                    styles: [":host{height:100%;width:100%;position:absolute;overflow:hidden}"]
                },] }
    ];
    NgxWidgetGridComponent.ctorParameters = function () { return [
        { type: core.ElementRef },
        { type: core.ChangeDetectorRef },
        { type: core.Renderer2 }
    ]; };
    NgxWidgetGridComponent.propDecorators = {
        showGrid: [{ type: core.Input }],
        widgetPositionChange: [{ type: core.Output }],
        gridFull: [{ type: core.Output }],
        widgetComponents: [{ type: core.ContentChildren, args: [NgxWidgetComponent,] }],
        rows: [{ type: core.Input }],
        columns: [{ type: core.Input }],
        highlightNextPosition: [{ type: core.Input }]
    };

    var NgxGridOverlayComponent = /** @class */ (function () {
        function NgxGridOverlayComponent(sanitizer) {
            this.sanitizer = sanitizer;
            this.activeHighlight = null;
            this.gridRows = [];
            this.gridCols = [];
            this._showGrid = false;
        }
        Object.defineProperty(NgxGridOverlayComponent.prototype, "renderer", {
            get: function () {
                return this._renderer;
            },
            set: function (renderer) {
                this._renderer = renderer;
                if (Utils.isDefined(renderer)) {
                    this.updateGridLines(renderer, this.showGrid);
                }
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgxGridOverlayComponent.prototype, "rows", {
            set: function (rows) {
                this.updateGridLines(this.renderer, this.showGrid);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgxGridOverlayComponent.prototype, "cols", {
            set: function (rows) {
                this.updateGridLines(this.renderer, this.showGrid);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgxGridOverlayComponent.prototype, "highlight", {
            get: function () {
                return this._highlight;
            },
            set: function (highlight) {
                this._highlight = highlight;
                this.clearHighlight();
                if (highlight) {
                    this.highlightArea(highlight, this.renderer);
                }
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgxGridOverlayComponent.prototype, "showGrid", {
            get: function () {
                return this._showGrid;
            },
            set: function (showGrid) {
                this._showGrid = showGrid;
                this.updateGridLines(this.renderer, showGrid);
            },
            enumerable: false,
            configurable: true
        });
        NgxGridOverlayComponent.prototype.updateGridLines = function (renderer, showGrid) {
            this.clearGridLines();
            if (showGrid) {
                this.showGridLines(renderer);
            }
        };
        NgxGridOverlayComponent.prototype.clearHighlight = function () {
            this.activeHighlight = null;
        };
        NgxGridOverlayComponent.prototype.clearGridLines = function () {
            this.gridRows.splice(0);
            this.gridCols.splice(0);
        };
        NgxGridOverlayComponent.prototype.highlightArea = function (area, renderer) {
            var cellSize = renderer.grid.cellSize;
            var cellHeight = cellSize.height;
            var cellWidth = cellSize.width;
            this.activeHighlight = {
                x: (area.left - 1) * cellWidth + '%',
                y: (area.top - 1) * cellHeight + '%',
                height: area.height * cellHeight + '%',
                width: area.width * cellWidth + '%'
            };
            this.sanitizer.bypassSecurityTrustStyle(this.activeHighlight);
        };
        NgxGridOverlayComponent.prototype.showGridLines = function (renderer) {
            if (renderer) {
                var cellHeight = renderer.grid.cellSize.height;
                var cellWidth = renderer.grid.cellSize.width;
                var height = cellHeight + '%';
                var width = cellWidth + '%';
                var rows = renderer.grid.rows;
                var cols = renderer.grid.columns;
                for (var i = 1; i < rows; i += 2) {
                    var y = void 0, h = void 0, row = void 0;
                    y = (i * cellHeight) + '%';
                    h = 'calc(' + height + ' - 1px)';
                    row = {
                        y: this.sanitizer.bypassSecurityTrustStyle(y),
                        height: this.sanitizer.bypassSecurityTrustStyle(h)
                    };
                    this.gridRows.push(row);
                }
                for (var i = 1; i < cols; i += 2) {
                    var x = void 0, w = void 0, col = void 0;
                    x = (i * cellWidth) + '%';
                    w = 'calc(' + width + ' - 1px)';
                    col = {
                        x: this.sanitizer.bypassSecurityTrustStyle(x),
                        width: this.sanitizer.bypassSecurityTrustStyle(w)
                    };
                    this.gridCols.push(col);
                }
            }
        };
        return NgxGridOverlayComponent;
    }());
    NgxGridOverlayComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'ngx-grid-overlay',
                    template: "<div class=\"wg-grid-overlay\">\n  <div *ngFor=\"let row of gridRows\"\n       class=\"wg-preview-item wg-preview-row\"\n       [style.height]=\"row.height\"\n       [style.top]=\"row.y\"></div>\n  <div *ngFor=\"let column of gridCols\"\n       class=\"wg-preview-item wg-preview-column\"\n       [style.width]=\"column.width\"\n       [style.left]=\"column.x\"></div>\n  <div *ngIf=\"activeHighlight\"\n       class=\"wg-preview-item wg-preview-highlight\"\n       [style.top]=\"activeHighlight.y\"\n       [style.height]=\"activeHighlight.height\"\n       [style.left]=\"activeHighlight.x\"\n       [style.width]=\"activeHighlight.width\"></div>\n</div>\n",
                    changeDetection: core.ChangeDetectionStrategy.OnPush,
                    styles: [".wg-grid-overlay{overflow:hidden}.wg-grid-overlay *{position:absolute;height:100%;width:100%}.wg-grid-overlay .wg-preview-item{position:absolute;display:inline-block}.wg-grid-overlay .wg-preview-item.wg-preview-row{width:100%;border-top:1px dotted #d7d7d7;border-bottom:1px solid #f0f0f0}.wg-grid-overlay .wg-preview-item.wg-preview-column{height:100%;border-left:1px dotted #d7d7d7;border-right:1px solid #f0f0f0}.wg-grid-overlay .wg-preview-item.wg-preview-highlight{background-color:rgba(0,113,188,.2);z-index:1}"]
                },] }
    ];
    NgxGridOverlayComponent.ctorParameters = function () { return [
        { type: platformBrowser.DomSanitizer }
    ]; };
    NgxGridOverlayComponent.propDecorators = {
        renderer: [{ type: core.Input }],
        rows: [{ type: core.Input }],
        cols: [{ type: core.Input }],
        highlight: [{ type: core.Input }],
        showGrid: [{ type: core.Input }]
    };

    var PathIterator = /** @class */ (function () {
        function PathIterator(start, end) {
            this._currPos = null;
            this._nextPos = null;
            if (!start) {
                console.error('Start not present for Path Iterator');
                return;
            }
            if (!end) {
                console.error('End not present for Path Iterator');
                return;
            }
            this._start = start;
            this._heightDelta = end.top - start.top;
            this._widthDelta = end.left - start.left;
            this._steps = Math.max(Math.abs(this._heightDelta), Math.abs(this._widthDelta));
            this._currStep = 0;
            this._currPos = null;
            this._nextPos = new Cell(start.top, start.left);
        }
        PathIterator.prototype.next = function () {
            this._currPos = this._nextPos;
            if (this._currStep < this._steps) {
                this._currStep++;
                var currTopDelta = Math.round((this._currStep / this._steps) * this._heightDelta);
                var currLeftDelta = Math.round((this._currStep / this._steps) * this._widthDelta);
                this._nextPos = new Cell(this._start.top + currTopDelta, this._start.left + currLeftDelta);
            }
            else {
                this._nextPos = null;
            }
            return this._currPos;
        };
        PathIterator.prototype.hasNext = function () {
            return this._nextPos !== null;
        };
        return PathIterator;
    }());

    var NgxWidgetMoverDirective = /** @class */ (function () {
        function NgxWidgetMoverDirective(el, renderer, gridCmp, widgetCmp) {
            this.el = el;
            this.renderer = renderer;
            this.gridCmp = gridCmp;
            this.widgetCmp = widgetCmp;
            this.enableDrag = null;
            this._onMoveListener = this.onMove.bind(this);
            this._onUpListener = this.onUp.bind(this);
            this.ngxWidgetMover = false;
        }
        NgxWidgetMoverDirective.prototype.onDown = function (event) {
            if (event.button === 0) {
                event.preventDefault();
                this.renderer.addClass(this.widgetCmp.getEl().nativeElement, 'wg-moving');
                var widgetContainer = this.widgetCmp.getEl().nativeElement;
                this.startPosition = this.gridCmp.getWidgetPosition(this.widgetCmp);
                this.startRender = {
                    top: widgetContainer.offsetTop,
                    left: widgetContainer.offsetLeft,
                    height: widgetContainer.clientHeight,
                    width: widgetContainer.clientWidth
                }; // pixel values
                var eventOffsetX = event.offsetX || event.layerX;
                var eventOffsetY = event.offsetY || event.layerY;
                this.desiredPosition = { top: this.startRender.top, left: this.startRender.left };
                this.moverOffset = new Rectangle({
                    top: eventOffsetY + this.el.nativeElement.offsetTop || 0,
                    left: eventOffsetX + this.el.nativeElement.offsetLeft || 0
                });
                this.gridPositions = this.gridCmp.getGridRectangle();
                this.cellHeight = (this.gridCmp.grid.cellSize.height / 100) * this.gridPositions.height;
                this.cellWidth = (this.gridCmp.grid.cellSize.width / 100) * this.gridPositions.width;
                this.enableDrag = this.widgetCmp.getConfig().id;
                this.renderer.setStyle(this.widgetCmp.getEl().nativeElement, 'z-index', this.ngxWidgetMover ? 0 : 100);
                if (typeof PointerEvent !== 'undefined') {
                    window.addEventListener('pointermove', this._onMoveListener);
                    window.addEventListener('pointerup', this._onUpListener);
                }
                else {
                    window.addEventListener('mousemove', this._onMoveListener);
                    window.addEventListener('mouseup', this._onUpListener);
                }
            }
        };
        NgxWidgetMoverDirective.prototype.onMove = function (event) {
            if (this.enableDrag === this.widgetCmp.getConfig().id) {
                event.preventDefault();
                var eventClientX = event.clientX;
                var eventClientY = event.clientY;
                var startRender = this.startRender;
                var gridDimensions = this.gridPositions;
                var desiredPosition = this.desiredPosition;
                // normalize the drag position
                var dragPositionX = Math.round(eventClientX) - gridDimensions.left, dragPositionY = Math.round(eventClientY) - gridDimensions.top;
                desiredPosition.top = Math.min(Math.max(dragPositionY - this.moverOffset.top, 0), gridDimensions.height - startRender.height - 1);
                desiredPosition.left = Math.min(Math.max(dragPositionX - this.moverOffset.left, 0), gridDimensions.width - startRender.width - 1);
                var currentFinalPos = this.determineFinalPos(this.startPosition, desiredPosition, this.startRender, this.cellHeight, this.cellWidth);
                this.gridCmp.highlightArea(currentFinalPos);
                this.renderer.setStyle(this.widgetCmp.getEl().nativeElement, 'top', desiredPosition.top + 'px');
                this.renderer.setStyle(this.widgetCmp.getEl().nativeElement, 'left', desiredPosition.left + 'px');
            }
        };
        NgxWidgetMoverDirective.prototype.onUp = function (event) {
            if (this.enableDrag === this.widgetCmp.getConfig().id) {
                event.preventDefault();
                var eventClientX = event.clientX;
                var eventClientY = event.clientY;
                var startRender = this.startRender;
                var gridDimensions = this.gridPositions;
                var desiredPosition = this.desiredPosition;
                // normalize the drag position
                var dragPositionX = Math.round(eventClientX) - gridDimensions.left, dragPositionY = Math.round(eventClientY) - gridDimensions.top;
                desiredPosition.top = Math.min(Math.max(dragPositionY - this.moverOffset.top, 0), gridDimensions.height - startRender.height - 1);
                desiredPosition.left = Math.min(Math.max(dragPositionX - this.moverOffset.left, 0), gridDimensions.width - startRender.width - 1);
                var anchorTop = this.getAnchor(Math.max(dragPositionY, 0), this.cellHeight, 1);
                var anchorLeft = this.getAnchor(Math.max(dragPositionX, 0), this.cellWidth, 1);
                var dropPosition = this.gridCmp.rasterizeCoords(anchorLeft, anchorTop);
                var obstructingWidgetId = this.gridCmp.areaObstructor(dropPosition);
                var finalPos = void 0;
                if (obstructingWidgetId && this.ngxWidgetMover) {
                    var obstructingWidgetCmp = this.gridCmp.getWidgetById(obstructingWidgetId);
                    var obstructingWidgetPosition = this.gridCmp.getWidgetPositionByWidgetId(obstructingWidgetId);
                    var draggedWidgetPosition = this.widgetCmp.position;
                    this.widgetCmp.position = obstructingWidgetPosition;
                    this.gridCmp.updateWidget(this.widgetCmp, true);
                    obstructingWidgetCmp.position = draggedWidgetPosition;
                    this.gridCmp.updateWidget(obstructingWidgetCmp, true);
                }
                else {
                    finalPos = this.determineFinalPos(this.startPosition, desiredPosition, this.startRender, this.cellHeight, this.cellWidth);
                    this.widgetCmp.position = finalPos;
                    this.gridCmp.updateWidget(this.widgetCmp, false);
                }
                this.gridCmp.resetHighlights();
                this.renderer.removeClass(this.widgetCmp.getEl().nativeElement, 'wg-moving');
                this.renderer.removeStyle(this.widgetCmp.getEl().nativeElement, 'z-index');
                this.enableDrag = null;
            }
            if (typeof PointerEvent !== 'undefined') {
                window.removeEventListener('pointermove', this._onMoveListener);
                window.removeEventListener('pointerup', this._onUpListener);
            }
            else {
                window.removeEventListener('mousemove', this._onMoveListener);
                window.removeEventListener('mouseup', this._onUpListener);
            }
        };
        NgxWidgetMoverDirective.prototype.getAnchor = function (val, cellWOrH, marginFactor) {
            if (marginFactor === void 0) { marginFactor = 2; }
            return (val % cellWOrH) > (cellWOrH / marginFactor) ? val + Math.floor(cellWOrH) : val;
        };
        NgxWidgetMoverDirective.prototype.determineFinalPos = function (startPos, desiredPos, startRender, cellHt, cellWd) {
            if (startRender.top === desiredPos.top && startRender.left === desiredPos.left) {
                return startPos;
            }
            var anchorTop = this.getAnchor(desiredPos.top, cellHt);
            var anchorLeft = this.getAnchor(desiredPos.left, cellWd);
            var movedDown = anchorTop >= startRender.top;
            var movedRight = anchorLeft >= startRender.left;
            var desiredFinalPosition = this.gridCmp.rasterizeCoords(anchorLeft, anchorTop);
            var path = new PathIterator(desiredFinalPosition, startPos);
            while (path && path.hasNext()) {
                var currPos = path.next();
                var targetArea = new Rectangle({
                    top: currPos.top,
                    left: currPos.left,
                    height: startPos.height,
                    width: startPos.width
                });
                var options = {
                    excludedArea: startPos,
                    fromBottom: movedDown,
                    fromRight: movedRight
                };
                // If widget swap is enabled and area is obstructed, show original position
                if (this.ngxWidgetMover && this.gridCmp.isAreaObstructed(targetArea, options)) {
                    return new Rectangle(startPos);
                }
                if (!this.gridCmp.isAreaObstructed(targetArea, options)) {
                    // try to get closer to the desired position by leaving the original path
                    var height = targetArea.height;
                    var width = targetArea.width;
                    if (desiredFinalPosition.top < targetArea.top) {
                        while (desiredFinalPosition.top <= (targetArea.top - 1)) {
                            var checkRect = new Rectangle({ top: targetArea.top - 1, left: targetArea.left, height: height, width: width });
                            var isRectVacant = !this.gridCmp.isAreaObstructed(checkRect, options);
                            if (isRectVacant) {
                                targetArea.top--;
                            }
                            else {
                                break;
                            }
                        }
                    }
                    else if (desiredFinalPosition.top > targetArea.top) {
                        while (desiredFinalPosition.top >= (targetArea.top + 1)) {
                            var checkRect = new Rectangle({ top: targetArea.top + 1, left: targetArea.left, height: height, width: width });
                            var isRectVacant = !this.gridCmp.isAreaObstructed(checkRect, options);
                            if (isRectVacant) {
                                targetArea.top++;
                            }
                            else {
                                break;
                            }
                        }
                    }
                    if (desiredFinalPosition.left < targetArea.left) {
                        while (desiredFinalPosition.left <= (targetArea.left - 1)) {
                            var checkRect = new Rectangle({ top: targetArea.top, left: targetArea.left - 1, height: height, width: width });
                            var isRectVacant = !this.gridCmp.isAreaObstructed(checkRect, options);
                            if (isRectVacant) {
                                targetArea.left--;
                            }
                            else {
                                break;
                            }
                        }
                    }
                    else if (desiredFinalPosition.left > targetArea.left) {
                        while (desiredFinalPosition.left >= (targetArea.left + 1)) {
                            var checkRect = new Rectangle({ top: targetArea.top, left: targetArea.left + 1, height: height, width: width });
                            var isRectVacant = !this.gridCmp.isAreaObstructed(checkRect, options);
                            if (isRectVacant) {
                                targetArea.left++;
                            }
                            else {
                                break;
                            }
                        }
                    }
                    return new Rectangle(targetArea);
                }
            }
            return new Rectangle(startPos);
        };
        return NgxWidgetMoverDirective;
    }());
    NgxWidgetMoverDirective.decorators = [
        { type: core.Directive, args: [{
                    selector: '[ngxWidgetMover]'
                },] }
    ];
    NgxWidgetMoverDirective.ctorParameters = function () { return [
        { type: core.ElementRef },
        { type: core.Renderer2 },
        { type: NgxWidgetGridComponent, decorators: [{ type: core.Inject, args: [core.forwardRef(function () { return NgxWidgetGridComponent; }),] }] },
        { type: NgxWidgetComponent, decorators: [{ type: core.Inject, args: [core.forwardRef(function () { return NgxWidgetComponent; }),] }] }
    ]; };
    NgxWidgetMoverDirective.propDecorators = {
        ngxWidgetMover: [{ type: core.Input }],
        onDown: [{ type: core.HostListener, args: ['mousedown', ['$event'],] }, { type: core.HostListener, args: ['pointerdown', ['$event'],] }]
    };

    var MIN_HEIGHT = 42;
    var MIN_WIDTH = 42;
    var NgxWidgetResizerDirective = /** @class */ (function () {
        function NgxWidgetResizerDirective(el, renderer, gridCmp, widgetCmp) {
            this.el = el;
            this.renderer = renderer;
            this.gridCmp = gridCmp;
            this.widgetCmp = widgetCmp;
            this.moveUpAllowed = false;
            this.moveDownAllowed = false;
            this.moveLeftAllowed = false;
            this.moveRightAllowed = false;
            this.enableDrag = null;
            this._onMoveListener = this.onMove.bind(this);
            this._onUpListener = this.onUp.bind(this);
            this.parentContainer = this.el.nativeElement.parentElement;
        }
        Object.defineProperty(NgxWidgetResizerDirective.prototype, "resizeDirection", {
            get: function () {
                return this._resizeDirection;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgxWidgetResizerDirective.prototype, "ngxWidgetResizer", {
            set: function (dir) {
                this._resizeDirection = dir;
                this.moveUpAllowed = false;
                this.moveDownAllowed = false;
                this.moveLeftAllowed = false;
                this.moveRightAllowed = false;
                switch (dir) {
                    case RESIZE_DIRECTIONS.top:
                        this.moveUpAllowed = true;
                        break;
                    case RESIZE_DIRECTIONS.left:
                        this.moveLeftAllowed = true;
                        break;
                    case RESIZE_DIRECTIONS.bottom:
                        this.moveDownAllowed = true;
                        break;
                    case RESIZE_DIRECTIONS.right:
                        this.moveRightAllowed = true;
                        break;
                    case RESIZE_DIRECTIONS.topLeft:
                        this.moveUpAllowed = true;
                        this.moveLeftAllowed = true;
                        break;
                    case RESIZE_DIRECTIONS.topRight:
                        this.moveUpAllowed = true;
                        this.moveRightAllowed = true;
                        break;
                    case RESIZE_DIRECTIONS.bottomLeft:
                        this.moveDownAllowed = true;
                        this.moveLeftAllowed = true;
                        break;
                    case RESIZE_DIRECTIONS.bottomRight:
                        this.moveDownAllowed = true;
                        this.moveRightAllowed = true;
                        break;
                    default:
                }
            },
            enumerable: false,
            configurable: true
        });
        NgxWidgetResizerDirective.prototype.onDown = function (event) {
            event.preventDefault();
            this.enableDrag = this.widgetCmp.getConfig().id;
            this.renderer.addClass(this.widgetCmp.getEl().nativeElement, 'wg-resizing');
            this.renderer.addClass(this.el.nativeElement, 'dragging');
            this.startPosition = this.gridCmp.getWidgetPosition(this.widgetCmp);
            this.startRender = {
                top: Math.ceil(this.widgetCmp.getEl().nativeElement.offsetTop),
                left: Math.ceil(this.widgetCmp.getEl().nativeElement.offsetLeft),
                height: Math.floor(this.parentContainer.offsetHeight),
                width: Math.floor(this.parentContainer.offsetWidth)
            }; // pixel values
            this.startRender.bottom = this.startRender.top + this.startRender.height;
            this.startRender.right = this.startRender.left + this.startRender.width;
            var eventOffsetX = event.offsetX || event.layerX;
            var eventOffsetY = event.offsetY || event.layerY;
            this.delta = { top: 0, right: 0, bottom: 0, left: 0 };
            this.draggerOffset = {
                top: eventOffsetY,
                left: eventOffsetX,
                bottom: eventOffsetY - this.el.nativeElement.offsetHeight,
                right: eventOffsetX - this.el.nativeElement.offsetWidth
            };
            this.gridPositions = this.gridCmp.getGridRectangle();
            if (typeof PointerEvent !== 'undefined') {
                window.addEventListener('pointermove', this._onMoveListener);
                window.addEventListener('pointerup', this._onUpListener);
            }
            else {
                window.addEventListener('mousemove', this._onMoveListener);
                window.addEventListener('mouseup', this._onUpListener);
            }
        };
        NgxWidgetResizerDirective.prototype.onMove = function (event) {
            if (this.enableDrag === this.widgetCmp.getConfig().id) {
                event.preventDefault();
                var eventClientX = event.clientX;
                var eventClientY = event.clientY;
                var gridDims = this.gridPositions;
                var startRender = this.startRender;
                // normalize the drag position
                var dragPositionX = Math.round(eventClientX) - gridDims.left;
                var dragPositionY = Math.round(eventClientY) - gridDims.top;
                var delta = this.delta;
                if (this.moveUpAllowed) {
                    delta.top = Math.min(Math.max(dragPositionY - this.draggerOffset.top, 0), gridDims.height) - startRender.top;
                    delta.top = Math.min(delta.top, startRender.height - MIN_HEIGHT);
                }
                else if (this.moveDownAllowed) {
                    delta.bottom = startRender.bottom - Math.min(Math.max(dragPositionY - this.draggerOffset.bottom, 0), gridDims.height);
                    delta.bottom = Math.min(delta.bottom, startRender.height - MIN_HEIGHT);
                }
                if (this.moveLeftAllowed) {
                    delta.left = Math.min(Math.max(dragPositionX - this.draggerOffset.left, 0), gridDims.width) - startRender.left;
                    delta.left = Math.min(delta.left, startRender.width - MIN_WIDTH);
                }
                else if (this.moveRightAllowed) {
                    delta.right = startRender.right - Math.min(Math.max(dragPositionX - this.draggerOffset.right, 0), gridDims.width);
                    delta.right = Math.min(delta.right, startRender.width - MIN_WIDTH);
                }
                var currentFinalPos = this.determineFinalPos();
                this.gridCmp.highlightArea(currentFinalPos);
                this.renderer.setStyle(this.parentContainer, 'top', this.delta.top + 'px');
                this.renderer.setStyle(this.parentContainer, 'left', this.delta.left + 'px');
                this.renderer.setStyle(this.parentContainer, 'bottom', this.delta.bottom + 'px');
                this.renderer.setStyle(this.parentContainer, 'right', this.delta.right + 'px');
            }
        };
        NgxWidgetResizerDirective.prototype.onUp = function (event) {
            if (this.enableDrag === this.widgetCmp.getConfig().id) {
                event.preventDefault();
                this.el.nativeElement.setAttribute('draggable', false);
                this.renderer.removeClass(this.el.nativeElement, 'dragging');
                this.renderer.removeClass(this.widgetCmp.getEl().nativeElement, 'wg-resizing');
                this.enableDrag = null;
                this.widgetCmp.position = this.determineFinalPos();
                this.gridCmp.updateWidget(this.widgetCmp, false);
                this.gridCmp.resetHighlights();
                // reset style
                this.renderer.removeClass(this.widgetCmp.getEl().nativeElement, 'wg-resizing');
                this.renderer.removeClass(this.el.nativeElement, 'dragging');
                this.renderer.setStyle(this.parentContainer, 'top', '');
                this.renderer.setStyle(this.parentContainer, 'left', '');
                this.renderer.setStyle(this.parentContainer, 'bottom', '');
                this.renderer.setStyle(this.parentContainer, 'right', '');
            }
            if (typeof PointerEvent !== 'undefined') {
                window.removeEventListener('pointermove', this._onMoveListener);
                window.removeEventListener('pointerup', this._onUpListener);
            }
            else {
                window.removeEventListener('mousemove', this._onMoveListener);
                window.removeEventListener('mouseup', this._onUpListener);
            }
        };
        NgxWidgetResizerDirective.prototype.findCollision = function (start, end, val, reverse) {
            if (reverse === void 0) { reverse = false; }
            var foundCollision = false;
            for (var i = start; i <= end; i++) {
                var checker = reverse ? this.gridCmp.isPointObstructed(i, val) : this.gridCmp.isPointObstructed(val, i);
                if (checker) {
                    foundCollision = true;
                    break;
                }
            }
            return foundCollision;
        };
        NgxWidgetResizerDirective.prototype.determineFinalPos = function () {
            var finalPos = new Rectangle();
            var startRender = this.startRender;
            var delta = this.delta;
            var requestedStartPoint = this.gridCmp.rasterizeCoords(startRender.left + delta.left + 1, startRender.top + delta.top + 1);
            var requestedEndPoint = this.gridCmp.rasterizeCoords(startRender.right - delta.right - 1, startRender.bottom - delta.bottom - 1);
            var requestedPos = {
                top: requestedStartPoint.top,
                left: requestedStartPoint.left,
                right: requestedEndPoint.left,
                bottom: requestedEndPoint.top
            };
            // determine a suitable final position (one that is not obstructed)
            var foundCollision;
            var start = Math.max(this.startPosition.left, requestedPos.left);
            var end = Math.min(this.startPosition.right, requestedPos.right);
            if (this.moveUpAllowed && requestedPos.top < this.startPosition.top) {
                finalPos.top = this.startPosition.top;
                while (finalPos.top > requestedPos.top) {
                    // check whether adding another row above would cause any conflict
                    foundCollision = this.findCollision(start, end, finalPos.top - 1);
                    if (foundCollision) {
                        break;
                    }
                    finalPos.top--; // add row above
                }
            }
            else if (this.moveDownAllowed && requestedPos.bottom > this.startPosition.bottom) {
                finalPos.top = finalPos.top || requestedPos.top;
                finalPos.height = this.startPosition.bottom + 1 - this.startPosition.top;
                while (finalPos.bottom < requestedPos.bottom) {
                    // check whether adding another row below would cause any conflict
                    foundCollision = this.findCollision(start, end, finalPos.bottom + 1);
                    if (foundCollision) {
                        break;
                    }
                    finalPos.height++; // add row below
                }
            }
            finalPos.top = finalPos.top || requestedPos.top;
            finalPos.height = finalPos.height || requestedPos.bottom + 1 - finalPos.top;
            if (this.moveLeftAllowed && requestedPos.left < this.startPosition.left) {
                finalPos.left = this.startPosition.left;
                while (finalPos.left > requestedPos.left) {
                    // check whether adding another column would cause any conflict
                    foundCollision = this.findCollision(finalPos.top, finalPos.bottom, finalPos.left - 1, true);
                    if (foundCollision) {
                        break;
                    }
                    finalPos.left--; // add column
                }
            }
            else if (this.moveRightAllowed && requestedPos.right > this.startPosition.right) {
                finalPos.left = finalPos.left || requestedPos.left;
                finalPos.width = this.startPosition.right + 1 - this.startPosition.left;
                while (finalPos.right < requestedPos.right) {
                    foundCollision = this.findCollision(finalPos.top, finalPos.bottom, finalPos.right + 1, true);
                    if (foundCollision) {
                        break;
                    }
                    finalPos.width++;
                }
            }
            finalPos.left = finalPos.left || requestedPos.left;
            finalPos.width = finalPos.width || requestedPos.right + 1 - finalPos.left;
            return finalPos;
        };
        return NgxWidgetResizerDirective;
    }());
    NgxWidgetResizerDirective.decorators = [
        { type: core.Directive, args: [{
                    selector: '[ngxWidgetResizer]'
                },] }
    ];
    NgxWidgetResizerDirective.ctorParameters = function () { return [
        { type: core.ElementRef },
        { type: core.Renderer2 },
        { type: NgxWidgetGridComponent, decorators: [{ type: core.Inject, args: [core.forwardRef(function () { return NgxWidgetGridComponent; }),] }] },
        { type: NgxWidgetComponent, decorators: [{ type: core.Inject, args: [core.forwardRef(function () { return NgxWidgetComponent; }),] }] }
    ]; };
    NgxWidgetResizerDirective.propDecorators = {
        ngxWidgetResizer: [{ type: core.Input }],
        onDown: [{ type: core.HostListener, args: ['pointerdown', ['$event'],] }, { type: core.HostListener, args: ['mousedown', ['$event'],] }]
    };

    var NgxWidgetGridModule = /** @class */ (function () {
        function NgxWidgetGridModule() {
        }
        return NgxWidgetGridModule;
    }());
    NgxWidgetGridModule.decorators = [
        { type: core.NgModule, args: [{
                    imports: [
                        common.CommonModule
                    ],
                    declarations: [
                        NgxGridOverlayComponent,
                        NgxWidgetComponent,
                        NgxWidgetGridComponent,
                        NgxWidgetMoverDirective,
                        NgxWidgetResizerDirective
                    ],
                    exports: [
                        NgxWidgetComponent,
                        NgxWidgetGridComponent
                    ]
                },] }
    ];

    /*
     * Public API Surface of ngx-widget-grid
     */

    /**
     * Generated bundle index. Do not edit.
     */

    exports.Cell = Cell;
    exports.CellSize = CellSize;
    exports.Grid = Grid;
    exports.GridRenderer = GridRenderer;
    exports.NgxGridOverlayComponent = NgxGridOverlayComponent;
    exports.NgxWidgetComponent = NgxWidgetComponent;
    exports.NgxWidgetGridComponent = NgxWidgetGridComponent;
    exports.NgxWidgetGridModule = NgxWidgetGridModule;
    exports.PathIterator = PathIterator;
    exports.Rectangle = Rectangle;
    exports.WidgetConfig = WidgetConfig;
    exports.ɵa = NgxWidgetMoverDirective;
    exports.ɵb = NgxWidgetResizerDirective;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ngx-widget-grid.umd.js.map
